package net.wanji.opt.servicev2.evaluation.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.constant.ServiceLevelEnum;
import net.wanji.opt.entity.eventoptimize.TEventOptimizeInfo;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemAndStrategyDay;
import net.wanji.opt.dao.mapper.evaluation.CrossProblemEvaluationMapper;
import net.wanji.opt.po.trend.EventInfoSimplePo;
import net.wanji.opt.servicev2.evaluation.EvaluationInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
@Slf4j
public class EvaluationInfoServiceImpl extends ServiceImpl<CrossDataHistMapper, CrossDataHistPO> implements EvaluationInfoService {

    @Autowired
    CrossDataHistMapper crossDataHistMapper;
    @Autowired
    CrossProblemEvaluationMapper crossProblemEvaluationMapper;

    @Override
    public Map<String,Object> getListEvaluationSummarize(String crossId, String startTime, String endTime, String contrastStartTime, String constrastEndTime) {

      try {
          //分析时间的数据
          List<CrossDataHistPO> analyseList = crossDataHistMapper.getListEvaluationanalyse(crossId, startTime, endTime);

          //比较时间的数据
          List<CrossDataHistPO> contrastList = crossDataHistMapper.getListEvaluationcontrast(crossId, contrastStartTime, constrastEndTime);
          //分析时间服务水平
         // List<CrossDataHistPO> ServiceLevelanalyseList = crossDataHistMapper.getEvaluationanalyseServiceLevel(crossId, startTime, endTime);

          //比较时间的服务水平
        //  List<CrossDataHistPO> contrastServiceLevelList = crossDataHistMapper.getServiceLevelEvaluationcontrast(crossId, contrastStartTime, constrastEndTime);
          //取出分析时间 的速度 延误 排队长度
          Double speed = 0.00;
          Integer delayTime = 0;
          Double queueLength = 0.00;
          //分析的交通流量
          int analyseFlow = 0;
          for (CrossDataHistPO analyselist : analyseList) {
              if (analyselist == null) {
                  continue; // 跳过空元素
              }
              speed = analyselist.getSpeed();
              delayTime = analyselist.getDelayTime();
              queueLength = analyselist.getQueueLength();
              analyseFlow = analyselist.getFlow();
          }
          //取出对比时间 的速度 延误 排队长度
          Double speedCompare = 0.00;
          Integer delayTimeCompare = 0;
          Double queueLengthCompare = 0.00;
          //对比的交通流量
          int contrastFlow = 0;
          for (CrossDataHistPO contrastlist : contrastList) {
              if (contrastlist == null) {
                  continue; // 跳过空元素
              }
              speedCompare = contrastlist.getSpeed();
              delayTimeCompare = contrastlist.getDelayTime();
              queueLengthCompare = contrastlist.getQueueLength();
              contrastFlow = contrastlist.getFlow();
          }

          // 计算速度变化百分比
          double speedChangePercent = calculatePercentage(speed, speedCompare);

          // 计算延误变化百分比
          double delayTimeChangePercent = calculatePercentage(delayTime, delayTimeCompare);

          // 计算排队长度变化百分比
          double queueLengthChangePercent = calculatePercentage(queueLength, queueLengthCompare);

          // 计算交通流量百分比
          double flow = calculatePercentage(analyseFlow, contrastFlow);
          //取出分析时间的服务水平
//          String analyseserviceLevel = getServiceLevel(ServiceLevelanalyseList);
          //取出对比时间的服务水平
//          String compareServiceLevel = getServiceLevel(contrastServiceLevelList);
          JSONObject jsonObject = new JSONObject();
          //平均速度
          jsonObject.put("speed", speedChangePercent + "%");
          //平均延误
          jsonObject.put("delay", delayTimeChangePercent + "%");
          //平均排队时长
          jsonObject.put("queueLength", queueLengthChangePercent + "%");
          // 计算交通流量百分比
          jsonObject.put("flow", flow + "%");
//          jsonObject.put("analyseserviceLevel", analyseserviceLevel);
//          jsonObject.put("compareServiceLevel", compareServiceLevel);

          return jsonObject;
      }catch (Exception e){
          log.error("{} getListEvaluationSummarize error", this.getClass().getSimpleName(), e);
          throw  e;
      }
    }
    /**
     * 计算变化百分比
     *
     * @param analyseValue 分析值
     * @param contrastValue 对比值
     * @return 变化百分比（保留两位小数）
     */
    private static double calculatePercentage(double analyseValue, double contrastValue) {
        // 防止除零错误
        if (contrastValue == 0) {
            return (int)analyseValue*100;
        }

        // 计算变化值（分析值 - 对比值）
        double difference = analyseValue - contrastValue;

        // 计算变化百分比，并保留两位小数(差值/对比值)
        return Math.round((difference / contrastValue) * 100 * 100.0) / 100.0;
    }

     /*获取服务水平*/
    public  String getServiceLevel(List<CrossDataHistPO> list){

        //按服务水平分组
        Map<String, List<CrossDataHistPO>> groupByServerLevel = list.stream().collect(Collectors.groupingBy(o -> o.getServiceLevel()));
        double total = 0;
        for (Map.Entry<String, List<CrossDataHistPO>> entry1 : groupByServerLevel.entrySet()) {
            String key = entry1.getKey();
            total += ServiceLevelEnum.getByType(key).getVal() * entry1.getValue().size();
        }
        //服务水平加权平均计算：给服务水平(等级得分*该等级的数量)累加/总数量
        int avgService = (int) Math.round(total/list.size());
        String serviceLevel = ServiceLevelEnum.getByVal(avgService)==null?"A":ServiceLevelEnum.getByVal(avgService).getType();

        return  serviceLevel;
    }
    /*
    * 溢出
    * */
    @Override
    public Map<String, Object> summaryEvaluation(String evenType,String currentAlgo,String crossId, String startTime, String endTime, String contrastStartTime, String constrastEndTime) {

         try {

             //分析时间
             List<EventInfoSimplePo> overFlowList = crossProblemEvaluationMapper.getOverflow(evenType,currentAlgo,crossId, startTime, endTime);

             //对比时间
             List<EventInfoSimplePo> contrastoverFlowList = crossProblemEvaluationMapper.getContrastOverflow(evenType,currentAlgo,crossId, contrastStartTime, constrastEndTime);

              //分析时间的查询数据集
             //分析溢出次数
             long overFlowcount=0;
             //分析溢出优化次数
             long overFlowTime =0;
             //分析溢出时长
             int fenxioverflowTime=0;
             for (EventInfoSimplePo fxList:overFlowList){
                 //分析溢出次数
                 overFlowcount = fxList.getEventCount();
                 //分析溢出优化次数
                 overFlowTime = fxList.getOptimizecount();

                 fenxioverflowTime=fxList.getDuration();
             }

             //对比时间的查询数据集
             //对比溢出次数
             long ontrastoverFlowcount=0;
             //对比溢出优化次数
             long contrastoverFlowTime=0;
             //对比溢出时长
             int duibioverflowTime=0;
             for (EventInfoSimplePo fxList:contrastoverFlowList){
                 //对比溢出次数
                 ontrastoverFlowcount = fxList.getEventCount();
                 //对比溢出优化次数
                 contrastoverFlowTime = fxList.getOptimizecount();

                 duibioverflowTime=fxList.getDuration();
             }

             //计算溢出次数百分比
             double overFlowCishujisuan = calculatePercentage(overFlowcount, ontrastoverFlowcount);

             //计算溢时长百分比
             double overFlowtimejisuan = calculatePercentage(fenxioverflowTime, duibioverflowTime);


             //分析时间的数据t_cross_data_hist
             List<CrossDataHistPO> analyseList = crossDataHistMapper.getListEvaluationanalyse(crossId, startTime, endTime);

             //比较时间的数据t_cross_data_hist
             List<CrossDataHistPO> contrastList = crossDataHistMapper.getListEvaluationcontrast(crossId, contrastStartTime, constrastEndTime);
             //溢出分析的延时时间
             Integer overFlowdelayTime = 0;
             for (CrossDataHistPO analyselist : analyseList) {
                 if (analyselist == null) {
                     continue; // 跳过空元素
                 }
                 overFlowdelayTime = analyselist.getDelayTime();
             }

             //溢出对比的延时时间
             Integer conTrastoverFlowdelayTime = 0;
             for (CrossDataHistPO contrastlist : contrastList) {
                 if (contrastlist == null) {
                     continue; // 跳过空元素
                 }
                 conTrastoverFlowdelayTime = contrastlist.getDelayTime();
             }
             //溢出的平均延误时间计算
             double overFlowDelayjisuan = calculatePercentage(overFlowdelayTime, conTrastoverFlowdelayTime);

             JSONObject jsonObject = new JSONObject();
             //溢出次数
             jsonObject.put("analyOverFlowTime", overFlowcount);
             //溢出优化次数
             jsonObject.put("analyOverFlowOptimize", overFlowTime);
             //对比溢出次数
             jsonObject.put("contrastOverFlowTime", ontrastoverFlowcount);
             //对比的溢出优化次数
             jsonObject.put("contrastOverFlowOptimize", contrastoverFlowTime);
             //溢出次数分析对比计算
             jsonObject.put("OverFlowTimeCalculate", overFlowCishujisuan + "%");
             //溢出时长分析对比计算
             jsonObject.put("OverFlowHourCalculate", overFlowtimejisuan + "%");
             //溢出平均延误
             jsonObject.put("OverFlowavgdelay", overFlowDelayjisuan + "%");

             return jsonObject;
         }
         catch (Exception e){
             log.error("{} summaryEvaluation error", this.getClass().getSimpleName(), e);
             throw  e;
         }
    }


      /*失衡702
      *
      * */
    @Override
    public Map<String, Object> unbalanceEvaluation(String evenType,String currentAlgo,String crossId, String startTime, String endTime, String contrastStartTime, String constrastEndTime) {
        try {

            //分析时间
            List<EventInfoSimplePo> overFlowList = crossProblemEvaluationMapper.getOverflow(evenType,currentAlgo,crossId, startTime, endTime);

            //对比时间
            List<EventInfoSimplePo> contrastoverFlowList = crossProblemEvaluationMapper.getContrastOverflow(evenType,currentAlgo,crossId, contrastStartTime, constrastEndTime);

            //分析时间结果集
            //分析失衡次数
            long unbalancecount=0;
            //分析失衡优化次数
            long unbalanceTime=0;
            //分析失衡时长
            int fenxiunbalanceTime=0;
            for (EventInfoSimplePo fxList:overFlowList){
                //分析次数
                unbalancecount = fxList.getEventCount();
                //分析优化次数
                unbalanceTime = fxList.getOptimizecount();

                fenxiunbalanceTime=fxList.getDuration();
            }
             //对比时间结果集
            //对比失衡次数
            long contrastunbalancecount=0;
            //对比失衡优化次数
            long contrastunbalanceTime=0;
            //对比失衡时长
            int duibiunbalanceTime=0;
            for (EventInfoSimplePo fxList:contrastoverFlowList){
                //对比次数
                contrastunbalancecount = fxList.getEventCount();
                //对比优化次数
                contrastunbalanceTime = fxList.getOptimizecount();

                duibiunbalanceTime=fxList.getDuration();
            }

            //计算失衡次数百分比
            double unbalanceCishujisuan = calculatePercentage(unbalancecount, contrastunbalancecount);

            //计算失衡时长百分比
            double unbalancetimejisuan = calculatePercentage(fenxiunbalanceTime, duibiunbalanceTime);


            //分析时间路口平均停车次数t_cross_data_hist
            List<CrossDataHistPO> analyseList = crossDataHistMapper.getListEvaluationanalyse(crossId, startTime, endTime);

            //比较时间的数据路口平均停车次数t_cross_data_hist
            List<CrossDataHistPO> contrastList = crossDataHistMapper.getListEvaluationcontrast(crossId, contrastStartTime, constrastEndTime);
            //失衡分析时间的stop_times路口平均停车次数
            Double unbalanceStopTimes = 0.0;
            for (CrossDataHistPO analyselist : analyseList) {
                if (analyselist == null) {
                    continue; // 跳过空元素
                }
                unbalanceStopTimes = analyselist.getStopTimes();
            }

            //失衡对比的时间stop_times路口平均停车次数
            Double conTrastUnbalanceStopTimes = 0.0;
            for (CrossDataHistPO contrastlist : contrastList) {
                if (contrastlist == null) {
                    continue; // 跳过空元素
                }
                conTrastUnbalanceStopTimes = contrastlist.getStopTimes();
            }
            //失衡的平均停车次数计算
            double unbalanceStopTimesjisuan = calculatePercentage(unbalanceStopTimes, conTrastUnbalanceStopTimes);

            JSONObject jsonObject = new JSONObject();
            //分析失衡次数
            jsonObject.put("unbalancecount", unbalancecount);
            //分析失衡优化次数
            jsonObject.put("unbalanceTime", unbalanceTime);
            //对比失衡次数
            jsonObject.put("ontrastunbalancecount", contrastunbalancecount);
            //对比的失衡的优化次数
            jsonObject.put("contrastunbalanceTime", contrastunbalanceTime);
            //失衡次数分析对比计算
            jsonObject.put("unbalanceCishucalculate", unbalanceCishujisuan + "%");
            //失衡时长分析对比计算
            jsonObject.put("unbalancetimecalculate", unbalancetimejisuan + "%");
            //失衡平均停车次数
            jsonObject.put("unbalanceStopTimescalculate", unbalanceStopTimesjisuan + "%");

            return jsonObject;
        }catch (Exception e){
            log.error("{} unbalanceEvaluation error", this.getClass().getSimpleName(), e);
            throw  e;
        }

    }

   /*
   *  空放701
   * */
    @Override
    public Map<String, Object> idleEvaluation(String evenType,String currentAlgo,String crossId, String startTime, String endTime, String contrastStartTime, String constrastEndTime) {

        try {

            //分析时间
            List<EventInfoSimplePo> overFlowList = crossProblemEvaluationMapper.getOverflow(evenType,currentAlgo,crossId, startTime, endTime);

            //对比时间
            List<EventInfoSimplePo> contrastoverFlowList = crossProblemEvaluationMapper.getContrastOverflow(evenType,currentAlgo,crossId, contrastStartTime, constrastEndTime);


            //分析时间结果集
            //分析时间空放次数
            long dileCount=0;
            //分析时间空放优化次数
            long dileTime=0;
            //分析空放时长
            int fenxiDileTime=0;
            for (EventInfoSimplePo fxList:overFlowList){
                //分析次数
                dileCount = fxList.getEventCount();
                //分析优化次数
                dileTime = fxList.getOptimizecount();

                fenxiDileTime=fxList.getDuration();
            }
            //对比时间结果集
            //对比时间空放次数
            long contrastDileCount=0;
            //对比时间空放优化次数
            long contrastDileTime=0;
            //对比空放时长
            int duibiDileTime=0;
            for (EventInfoSimplePo fxList:contrastoverFlowList){
                //对比次数
                contrastDileCount = fxList.getEventCount();
                //对比优化次数
                contrastDileTime = fxList.getOptimizecount();

                duibiDileTime=fxList.getDuration();
            }
            //计算空放次数百分比
            double dileCishujisuan = calculatePercentage(dileCount, contrastDileCount);

            //计算空放时长百分比
            double dileTimejisuan = calculatePercentage(fenxiDileTime, duibiDileTime);


            //分析时间路口平均速度t_cross_data_hist
            List<CrossDataHistPO> analyseList = crossDataHistMapper.getListEvaluationanalyse(crossId, startTime, endTime);

            //比较时间的数据路口平均速度t_cross_data_hist
            List<CrossDataHistPO> contrastList = crossDataHistMapper.getListEvaluationcontrast(crossId, contrastStartTime, constrastEndTime);
            //空放分析时间的Speed路口平均速度
            Double dileSpeed = 0.0;
            for (CrossDataHistPO analyselist : analyseList) {
                if (analyselist == null) {
                    continue; // 跳过空元素
                }
                dileSpeed = analyselist.getSpeed();
            }

            //空放对比的时间平均速度
            Double conTrastDileSpeed = 0.0;
            for (CrossDataHistPO contrastlist : contrastList) {
                if (contrastlist == null) {
                    continue; // 跳过空元素
                }
                conTrastDileSpeed = contrastlist.getSpeed();
            }
            //空放的平均速度
            double dileSpeedjisuan = calculatePercentage(dileSpeed, conTrastDileSpeed);

            JSONObject jsonObject = new JSONObject();
            //分析时间空放次数
            jsonObject.put("dileCount", dileCount);
            //分析时间空放优化次数
            jsonObject.put("dileTime", dileTime);
            //对比时间空放的次数
            jsonObject.put("contrastDileCount", contrastDileCount);
            //对比的空放的优化次数
            jsonObject.put("contrastDileTime", contrastDileTime);
            //空放次数分析对比计算
            jsonObject.put("dileCishucalculate", dileCishujisuan + "%");
            //空放时长分析对比计算
            jsonObject.put("dileTimecalculate", dileTimejisuan + "%");
            //空放评价速度对比计算
            jsonObject.put("dileSpeedcalculate", dileSpeedjisuan + "%");

            return jsonObject;
        }catch (Exception e){
            log.error("{} idleEvaluation error", this.getClass().getSimpleName(), e);
            throw  e;
        }


    }
}
